/*  main_patch.c
 *
 *  Based on GooCanvas Demo ; test program
 *  Copyright (C) 2006 - 2010 Damon Chaplin, Alexis Vasic
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

#include <stdlib.h>
#include <cairo.h>
#include <goocanvas.h>
#include "goopatch.h"

static gboolean
on_motion_notify (GooPatch *Patch,
		  GooPatch *Target,
		  GdkEventMotion *event,
		  gpointer data);

static gboolean
on_button_release (GooPatch *Patch,
		   GooPatch *Target,
		   GdkEventButton *event,
		   gpointer data);

static gboolean
on_button_press (GooPatch *Patch,
		 GooPatch *Target,
		 GdkEventButton *event,
		 gpointer data);

static gboolean
on_delete_event  (GtkWidget	*window,
		  GdkEvent	*event,
		  gpointer	unused_data);
	    
static gboolean dragging = FALSE;
static double drag_x, drag_y;
  
  
int
main (int argc, char *argv[])
{
    GtkWidget *window, *scrolled_win, *canvas;
    GooPatch		*rect_patch;

    /* Initialize GTK+. */
    gtk_set_locale ();
    gtk_init (&argc, &argv);

    /* Create the window and widgets. */
    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    gtk_window_set_default_size (GTK_WINDOW (window), 640, 600);
    gtk_widget_show (window);

    g_signal_connect (window, "delete_event", (GtkSignalFunc) on_delete_event,
		      NULL);

    scrolled_win = gtk_scrolled_window_new (NULL, NULL);
    gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (scrolled_win),
					 GTK_SHADOW_IN);
    gtk_widget_show (scrolled_win);
    gtk_container_add (GTK_CONTAINER (window), scrolled_win);

    canvas = goo_canvas_new ();

    g_object_set (G_OBJECT (canvas),
		      "bounds-from-origin", FALSE,
		      "bounds-padding", 4.0,
		      "background-color", "grey",
 		      "has-tooltip", TRUE,
		      "redraw-when-scrolled", FALSE,
		      NULL);
    goo_canvas_set_bounds (GOO_CANVAS (canvas), 0, 0, 640, 600);

    gtk_widget_show (canvas);

    gtk_container_add (GTK_CONTAINER (scrolled_win), canvas);


    rect_patch = goo_patch_new (goo_canvas_get_root_item (GOO_CANVAS (canvas)),
							    2, 23, "000", "Opt and Coax Cap Volume",
							    50, 100);

    g_signal_connect (rect_patch->ItemGroup, "motion_notify_event",
		    G_CALLBACK (on_motion_notify), NULL);
    g_signal_connect (rect_patch->ItemGroup, "button_press_event",
		    G_CALLBACK (on_button_press), NULL);
    g_signal_connect (rect_patch->ItemGroup, "button_release_event",
		    G_CALLBACK (on_button_release), NULL);

    gtk_main ();

    return 0;
}

static gboolean
on_motion_notify (GooPatch *Patch,
		  GooPatch *Target,
		  GdkEventMotion *event,
		  gpointer data)
{
    if (dragging && (event->state & GDK_BUTTON1_MASK))
    {
	double new_x = event->x;
	double new_y = event->y;

	goo_canvas_item_translate ((GooCanvasItem *) Patch, new_x - drag_x, new_y - drag_y);
    }

    return TRUE;
}

static gboolean
on_button_release (GooPatch *Patch,
		   GooPatch *Target,
		   GdkEventButton *event,
		   gpointer data)
{
    GooCanvas *canvas;

    canvas = goo_canvas_item_get_canvas ((GooCanvasItem *) Patch);
    goo_canvas_pointer_ungrab (canvas, (GooCanvasItem *) Patch, event->time);
    dragging = FALSE;

    return TRUE;
}

static gboolean
on_button_press (GooPatch *Patch,
		 GooPatch *Target,
		 GdkEventButton *event,
		 gpointer data)
{
    GooCanvas *canvas;
    GdkCursor *fleur;
    GList *items, *elem;

    canvas = goo_canvas_item_get_canvas ((GooCanvasItem *) Patch);
    items = goo_canvas_get_items_at (canvas, event->x_root, event->y_root,
			   TRUE);

    for (elem = items; elem; elem = elem->next)

    g_list_free (items);

    switch (event->button)
    {
	case 1:
	if (event->state & GDK_SHIFT_MASK)
	{
	    goo_canvas_item_remove ((GooCanvasItem *) Patch);
	}
	else
	{
	    drag_x = event->x;
	    drag_y = event->y;

	    fleur = gdk_cursor_new (GDK_FLEUR);
	    goo_canvas_pointer_grab (canvas, (GooCanvasItem *) Patch,
				   GDK_POINTER_MOTION_MASK | GDK_POINTER_MOTION_HINT_MASK | GDK_BUTTON_RELEASE_MASK,
				   fleur,
				   event->time);
	    gdk_cursor_unref (fleur);
	    dragging = TRUE;
	}

	break;

	case 2:
	    goo_canvas_item_lower ((GooCanvasItem *) Patch, NULL);
	    break;

	case 3:
	    goo_canvas_item_raise ((GooCanvasItem *) Patch, NULL);
	    break;

	default:
	    break;
    }

  return TRUE;
}

static gboolean
on_delete_event (GtkWidget *window,
	       GdkEvent  *event,
	       gpointer   unused_data)
{
    exit (0);
}
